
# q-expansion of newform 39.4.j.c, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 39
weight = 4
poly_data = [27648, 0, 44100, 0, 14700, 0, 1645, 0, 70, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[96, 210, 0, 35, 0, 1, 0, 0, 0, 0], 192], [[0, 96, 210, 0, 35, 0, 1, 0, 0, 0], 192], [[14, 0, 1, 0, 0, 0, 0, 0, 0, 0], 1], [[1344, 2940, 96, 700, 0, 49, 0, 1, 0, 0], 192], [[24192, 15912, 12132, 9030, 918, 1309, 18, 64, 0, 1], 1152], [[24192, -15912, 12132, -9030, 918, -1309, 18, -64, 0, -1], 1152], [[48384, -20304, 42840, -12654, 7008, -1603, 366, -70, 6, -1], 1152], [[0, -20304, 0, -12654, 0, -1603, 0, -70, 0, -1], 576]]

hecke_ring_character_values = [[14, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [28, [1, 0, -1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 1, 0, 0, 0, 0, 0, 0], [0, 0, -3, 0, 0, 0, 0, 0, 0, 0], [-1, -1, 2, 0, 0, 0, 1, -1, 0, 0], [2, 0, 2, 0, 0, 0, 0, 0, 1, -1], [8, 0, -4, -5, -4, 2, 0, -1, -1, 0], [-6, -3, 17, -3, 0, -2, -1, 2, 1, 0], [21, 4, -21, 4, 0, 0, 0, 0, 1, 1], [12, 9, 12, -9, 4, 4, 3, 0, -1, 1], [0, 2, -12, -1, 0, 0, -2, 1, 3, -6], [0, -22, -99, 11, 0, 6, 6, -3, 0, 0], [22, 9, -44, 0, 6, -12, 3, -3, 0, -2], [-54, 0, 27, 36, 0, 0, 0, 12, -1, 0], [142, 0, -71, -32, 12, -6, 0, 8, -3, 0], [-74, -18, 74, -18, -6, 6, 10, -20, -1, -1], [-4, 47, 8, 0, 4, -8, -3, 3, 0, -1], [33, 3, 0, -6, 6, 0, 9, 9, -8, 4], [52, -26, 52, 26, -16, -16, 6, 0, 4, -4], [-275, -9, 275, -9, -4, 4, -3, 6, -1, -1], [212, 0, -106, -36, 28, -14, 0, -12, 3, 0], [108, -95, 108, 95, -4, -4, -1, 0, -5, 5], [57, -72, -114, 0, -10, 20, 0, 0, 0, 0], [110, -63, 0, 126, -2, 0, -13, -13, 4, -2], [172, 35, -344, 0, 10, -20, 37, -37, 0, -19], [272, 0, -136, 118, -8, 4, 0, -6, 10, 0], [-250, -81, -250, 81, 32, 32, -3, 0, 22, -22], [0, -262, -57, 131, 0, -24, 14, -7, -2, 4], [-610, -48, 0, 96, 0, 0, -16, -16, -2, 1], [0, 278, -252, -139, 0, 18, -38, 19, -11, 22], [154, 279, -308, 0, 10, -20, -27, 27, 0, 46], [-507, -14, 507, -14, 66, -66, 10, -20, 9, 9], [0, -102, 598, 51, 0, 50, -2, 1, -24, 48], [-36, 144, 0, -288, 0, 0, 32, 32, 44, -22], [-831, 202, -831, -202, 10, 10, -14, 0, -43, 43], [-662, -33, 662, -33, -66, 66, -3, 6, -48, -48], [281, -197, 281, 197, 22, 22, -3, 0, 2, -2], [24, 171, -48, 0, 62, -124, 81, -81, 0, 3], [569, -69, 0, 138, -42, 0, -39, -39, 50, -25], [-354, 63, -354, -63, 26, 26, -51, 0, 74, -74], [1360, 0, -680, -14, 112, -56, 0, 34, 10, 0], [726, 82, -726, 82, 42, -42, -46, 92, 34, 34], [0, -622, -1284, 311, 0, -126, -2, 1, 5, -10], [839, 360, 0, -720, -6, 0, -80, -80, 42, -21], [60, 286, -60, 286, 144, -144, -78, 156, 20, 20], [386, 0, -193, 288, 200, -100, 0, -24, -66, 0], [328, 0, -164, -44, 200, -100, 0, 96, -124, 0], [190, -240, -190, -240, 152, -152, -40, 80, -27, -27], [0, 6, 1522, -3, 0, -22, -158, 79, 2, -4], [-456, 0, 228, -270, -200, 100, 0, -198, -4, 0], [-600, -95, -600, 95, 122, 122, 47, 0, 49, -49], [-148, 594, 296, 0, 10, -20, 102, -102, 0, 2], [906, -276, 0, 552, 204, 0, 56, 56, -120, 60], [-1356, 233, 2712, 0, -104, 208, -61, 61, 0, -85], [-1023, -801, -1023, 801, -60, -60, 69, 0, -70, 70], [-1212, -146, 1212, -146, -96, 96, 78, -156, 22, 22], [0, -316, -369, 158, 0, -372, -20, 10, -59, 118], [0, -250, 1656, 125, 0, 348, -22, 11, -37, 74], [-1002, -80, 1002, -80, 66, -66, 152, -304, -50, -50], [44, 0, -22, -261, -4, 2, 0, 159, 24, 0], [3979, 30, -3979, 30, -258, 258, -6, 12, -49, -49], [745, -586, -1490, 0, 106, -212, -206, 206, 0, -13], [0, 1164, 742, -582, 0, -310, -412, 206, -47, 94], [35, -329, 35, 329, -184, -184, 85, 0, 82, -82], [1854, -414, -3708, 0, -230, 460, 198, -198, 0, -45], [-2544, 369, 0, -738, -396, 0, -17, -17, 90, -45], [2384, 75, 0, -150, 304, 0, 137, 137, -188, 94], [-695, 1205, 1390, 0, -216, 432, 167, -167, 0, 6], [1710, -693, 1710, 693, -6, -6, 81, 0, 140, -140], [209, 66, 0, -132, -122, 0, -82, -82, 144, -72], [-1812, -257, 1812, -257, -354, 354, -79, 158, -47, -47], [1804, 0, -902, 513, -292, 146, 0, -51, 94, 0], [-3222, 0, 1611, 1138, 440, -220, 0, 286, 35, 0], [3952, -613, -7904, 0, 52, -104, 73, -73, 0, 17], [0, 1200, -2474, -600, 0, 252, -160, 80, -49, 98], [-1253, -429, 1253, -429, 8, -8, -79, 158, -115, -115], [3972, 0, -1986, 1971, 524, -262, 0, 303, -22, 0], [2764, 958, 2764, -958, 0, 0, 118, 0, -42, 42], [543, -957, 0, 1914, -426, 0, -231, -231, -124, 62], [-1882, -189, -1882, 189, 38, 38, 393, 0, 22, -22], [522, 0, -261, -1394, -132, 66, 0, -18, -27, 0], [-2167, 504, -2167, -504, 120, 120, -456, 0, -188, 188], [0, 212, 756, -106, 0, -168, -68, 34, 130, -260], [-1193, -1503, 2386, 0, 362, -724, -165, 165, 0, -71], [8064, 0, -4032, 505, -600, 300, 0, -351, -153, 0], [-2915, -1062, 2915, -1062, -94, 94, 14, -28, -44, -44], [0, 1344, 3658, -672, 0, -516, -144, 72, -75, 150], [-588, 270, 0, -540, -264, 0, -122, -122, -292, 146], [2044, -176, 2044, 176, 64, 64, -400, 0, 74, -74], [-1550, 0, 775, -36, 676, -338, 0, 180, 118, 0], [-1623, -629, -1623, 629, -90, -90, 537, 0, -126, 126], [-4318, 252, 8636, 0, -28, 56, -12, 12, 0, -199], [-4704, -735, 0, 1470, 66, 0, -141, -141, 518, -259], [6160, 0, -3080, -170, -736, 368, 0, -474, -130, 0], [2276, 2619, 2276, -2619, 34, 34, 129, 0, 145, -145], [0, -250, 2412, 125, 0, 582, 18, -9, 189, -378], [-708, -1314, 1416, 0, 156, -312, -150, 150, 0, 192], [-4968, 253, 4968, 253, 744, -744, 3, -6, 57, 57], [-13082, 0, 6541, -1439, -448, 224, 0, -531, 26, 0], [-2697, -120, 0, 240, 708, 0, 300, 300, 110, -55], [0, 1482, -2900, -741, 0, -996, -562, 281, 285, -570], [-65, 279, 130, 0, -384, 768, 429, -429, 0, 193], [-2014, -996, 0, 1992, -114, 0, -148, -148, -430, 215], [8278, 0, -4139, -1967, 96, -48, 0, 797, 108, 0], [-1068, 958, 1068, 958, -96, 96, -474, 948, -8, -8], [0, -472, 918, 236, 0, 222, 224, -112, -136, 272], [-2632, -987, 0, 1974, 136, 0, 215, 215, 98, -49], [3027, 1035, -6054, 0, -652, 1304, 897, -897, 0, -90], [-7488, 0, 3744, -626, 280, -140, 0, 194, 652, 0], [3467, -442, -6934, 0, -676, 1352, -46, 46, 0, -263], [-10548, -1224, 0, 2448, -192, 0, -428, -428, -20, 10], [0, 1362, -5567, -681, 0, 398, -1162, 581, -68, 136], [8440, -468, -8440, -468, 336, -336, 324, -648, 140, 140], [-2386, 0, 1193, -1161, 52, -26, 0, -909, 207, 0], [-2583, 364, -2583, -364, -76, -76, 764, 0, 157, -157], [-3078, -801, 6156, 0, 402, -804, 237, -237, 0, -174], [-6526, 729, -6526, -729, 166, 166, 3, 0, -476, 476], [3801, -668, -3801, -668, 1458, -1458, -132, 264, -125, -125], [4666, -981, 4666, 981, -118, -118, 681, 0, -48, 48], [0, 296, -3336, -148, 0, -648, -648, 324, 58, -116], [0, -448, -9930, 224, 0, 426, -184, 92, 22, -44], [792, -2246, -792, -2246, -276, 276, -258, 516, -210, -210], [6902, 0, -3451, 4257, 848, -424, 0, -75, -147, 0], [0, 12, 7699, -6, 0, -642, 660, -330, -264, 528], [-570, 3048, 0, -6096, 702, 0, -44, -44, -508, 254], [8828, -2066, 8828, 2066, 120, 120, 326, 0, -372, 372], [19300, 0, -9650, 234, -676, 338, 0, 66, 721, 0], [9642, 1476, 0, -2952, 150, 0, -524, -524, -60, 30], [7215, -1701, 7215, 1701, -334, -334, 81, 0, 17, -17], [-13236, -1232, 13236, -1232, 1116, -1116, 136, -272, -370, -370], [-2146, 366, 0, -732, 580, 0, 378, 378, -146, 73], [-5419, 675, 10838, 0, -366, 732, 777, -777, 0, 743], [2392, 0, -1196, 3834, 120, -60, 0, 522, 340, 0], [-768, 0, 384, 2668, 392, -196, 0, 460, -1162, 0], [0, -2142, -17228, 1071, 0, 822, -858, 429, 221, -442], [0, -3360, -9758, 1680, 0, 404, 432, -216, 292, -584], [8232, 1174, 8232, -1174, -736, -736, -1138, 0, 22, -22], [-18456, 0, 9228, 2142, -268, 134, 0, 174, -790, 0], [-2796, -2852, -2796, 2852, -354, -354, 276, 0, 546, -546], [-3662, 4365, -3662, -4365, 6, 6, 255, 0, 594, -594], [2322, 814, -2322, 814, -384, 384, 366, -732, 444, 444], [0, 4820, -2391, -2410, 0, 204, -1372, 686, -345, 690], [2642, 9, -5284, 0, 350, -700, -573, 573, 0, 506], [-26120, 0, 13060, 436, -684, 342, 0, -196, -546, 0], [-5288, -2328, 5288, -2328, 816, -816, -168, 336, -264, -264], [-4556, -4330, 9112, 0, -240, 480, -694, 694, 0, 234], [0, 3162, 11257, -1581, 0, -1186, -1522, 761, -379, 758], [3300, -1634, 3300, 1634, -244, -244, -154, 0, -584, 584], [-13911, 117, 27822, 0, -492, 984, 303, -303, 0, -69], [19317, -1242, 0, 2484, -1050, 0, 850, 850, 1002, -501], [-12010, 36, 0, -72, -542, 0, -724, -724, 330, -165], [2012, 179, -4024, 0, 240, -480, 961, -961, 0, -681], [1607, -2772, 1607, 2772, -1354, -1354, -612, 0, -371, 371], [0, 1184, 1479, -592, 0, 1224, 1328, -664, 87, -174], [-14074, 3033, 0, -6066, -1802, 0, -61, -61, -584, 292], [0, -2836, 14580, 1418, 0, -1968, -660, 330, -228, 456], [0, -6252, -21248, 3126, 0, -172, 188, -94, 126, -252], [-8460, 1026, 0, -2052, -72, 0, 50, 50, 908, -454], [-5108, -702, 5108, -702, -1528, 1528, 1158, -2316, 434, 434], [10323, 1858, 10323, -1858, 304, 304, -50, 0, 995, -995], [27653, 2343, 0, -4686, 568, 0, 269, 269, -148, 74], [15780, -856, -31560, 0, -830, 1660, 176, -176, 0, -802], [-128, 0, 64, 1648, -160, 80, 0, 288, 68, 0], [8982, -2492, -8982, -2492, -1710, 1710, 1460, -2920, -40, -40], [-11928, 1881, 23856, 0, -58, 116, -741, 741, 0, 585], [21156, 2062, -21156, 2062, 888, -888, -146, 292, 270, 270], [-16106, 0, 8053, -2030, 196, -98, 0, 274, 91, 0], [1056, 3482, -2112, 0, -308, 616, 254, -254, 0, -616], [0, 5550, -16640, -2775, 0, -270, 218, -109, -305, 610], [3889, 1995, -3889, 1995, -2772, 2772, 473, -946, 409, 409]]
